import { MatchRequest } from '../../../domain/dtos/match.request';
import { Role } from '../../../domain/types/role.enum';
import { IDefaultParams } from '../../../domain/types/default-params.type';
import { MatchQuery } from '../../../queries/match.query';
import { AlgorithmType } from '../../../domain/types/algorithm.enum';
import { Frequency } from '../../../../ad/domain/types/frequency.enum';
import { Mode } from '../../../domain/types/mode.enum';

const defaultParams: IDefaultParams = {
  DEFAULT_UUID: '00000000-0000-0000-0000-000000000000',
  MARGIN_DURATION: 900,
  VALIDITY_DURATION: 365,
  DEFAULT_TIMEZONE: 'Europe/Paris',
  DEFAULT_SEATS: 3,
  DEFAULT_ALGORITHM_SETTINGS: {
    algorithm: AlgorithmType.CLASSIC,
    strict: false,
    remoteness: 15000,
    useProportion: true,
    proportion: 0.3,
    useAzimuth: true,
    azimuthMargin: 10,
    maxDetourDistanceRatio: 0.3,
    maxDetourDurationRatio: 0.3,
    georouterType: 'graphhopper',
    georouterUrl: 'http://localhost',
  },
};

const mockGeorouterCreator = {
  create: jest.fn().mockImplementation(),
};

const mockTimezoneFinder = {
  timezones: jest.fn().mockImplementation(() => ['Europe/Paris']),
};

const mockTimeConverter = {
  toUtcDate: jest.fn(),
};

describe('Match query', () => {
  it('should be defined', () => {
    const matchRequest: MatchRequest = new MatchRequest();
    matchRequest.departure = '2023-04-01 12:00';
    matchRequest.waypoints = [
      {
        lat: 49.440041,
        lon: 1.093912,
      },
      {
        lat: 50.630992,
        lon: 3.045432,
      },
    ];
    const matchQuery: MatchQuery = new MatchQuery(
      matchRequest,
      defaultParams,
      mockGeorouterCreator,
      mockTimezoneFinder,
      mockTimeConverter,
    );
    expect(matchQuery).toBeDefined();
    expect(matchQuery.mode).toBe(Mode.MATCH);
  });

  it('should create a query with publish and match mode', () => {
    const matchRequest: MatchRequest = new MatchRequest();
    matchRequest.departure = '2023-04-01 12:00';
    matchRequest.waypoints = [
      {
        lat: 49.440041,
        lon: 1.093912,
      },
      {
        lat: 50.630992,
        lon: 3.045432,
      },
    ];
    matchRequest.mode = Mode.PUBLISH_AND_MATCH;
    const matchQuery: MatchQuery = new MatchQuery(
      matchRequest,
      defaultParams,
      mockGeorouterCreator,
      mockTimezoneFinder,
      mockTimeConverter,
    );
    expect(matchQuery.mode).toBe(Mode.PUBLISH_AND_MATCH);
  });

  it('should create a query with excluded identifiers', () => {
    const matchRequest: MatchRequest = new MatchRequest();
    matchRequest.departure = '2023-04-01 12:00';
    matchRequest.waypoints = [
      {
        lat: 49.440041,
        lon: 1.093912,
      },
      {
        lat: 50.630992,
        lon: 3.045432,
      },
    ];
    matchRequest.uuid = '445aa6e4-99e4-4899-9456-3be8c3ada368';
    matchRequest.exclusions = [
      'eacf5e53-e63c-4551-860c-73f95b8a8895',
      'a4098161-13a9-4e55-8999-de134fbf89c4',
      'b18f7ffa-20b9-4a1a-89bc-e238ea8289f3',
    ];
    const matchQuery: MatchQuery = new MatchQuery(
      matchRequest,
      defaultParams,
      mockGeorouterCreator,
      mockTimezoneFinder,
      mockTimeConverter,
    );
    expect(matchQuery.exclusions.length).toBe(4);
  });

  it('should create a query with driver role only', () => {
    const matchRequest: MatchRequest = new MatchRequest();
    matchRequest.departure = '2023-04-01 12:00';
    matchRequest.waypoints = [
      {
        lat: 49.440041,
        lon: 1.093912,
      },
      {
        lat: 50.630992,
        lon: 3.045432,
      },
    ];
    matchRequest.driver = true;
    const matchQuery: MatchQuery = new MatchQuery(
      matchRequest,
      defaultParams,
      mockGeorouterCreator,
      mockTimezoneFinder,
      mockTimeConverter,
    );
    expect(matchQuery.roles).toEqual([Role.DRIVER]);
  });

  it('should create a query with passenger role only', () => {
    const matchRequest: MatchRequest = new MatchRequest();
    matchRequest.departure = '2023-04-01 12:00';
    matchRequest.waypoints = [
      {
        lat: 49.440041,
        lon: 1.093912,
      },
      {
        lat: 50.630992,
        lon: 3.045432,
      },
    ];
    matchRequest.passenger = true;
    const matchQuery: MatchQuery = new MatchQuery(
      matchRequest,
      defaultParams,
      mockGeorouterCreator,
      mockTimezoneFinder,
      mockTimeConverter,
    );
    expect(matchQuery.roles).toEqual([Role.PASSENGER]);
  });

  it('should create a query with driver and passenger roles', () => {
    const matchRequest: MatchRequest = new MatchRequest();
    matchRequest.departure = '2023-04-01 12:00';
    matchRequest.waypoints = [
      {
        lat: 49.440041,
        lon: 1.093912,
      },
      {
        lat: 50.630992,
        lon: 3.045432,
      },
    ];
    matchRequest.passenger = true;
    matchRequest.driver = true;
    const matchQuery: MatchQuery = new MatchQuery(
      matchRequest,
      defaultParams,
      mockGeorouterCreator,
      mockTimezoneFinder,
      mockTimeConverter,
    );
    expect(matchQuery.roles.length).toBe(2);
    expect(matchQuery.roles).toContain(Role.PASSENGER);
    expect(matchQuery.roles).toContain(Role.DRIVER);
  });

  it('should create a query with number of seats modified', () => {
    const matchRequest: MatchRequest = new MatchRequest();
    matchRequest.departure = '2023-04-01 12:00';
    matchRequest.waypoints = [
      {
        lat: 49.440041,
        lon: 1.093912,
      },
      {
        lat: 50.630992,
        lon: 3.045432,
      },
    ];
    matchRequest.seatsDriver = 1;
    matchRequest.seatsPassenger = 2;
    const matchQuery: MatchQuery = new MatchQuery(
      matchRequest,
      defaultParams,
      mockGeorouterCreator,
      mockTimezoneFinder,
      mockTimeConverter,
    );
    expect(matchQuery.requirement.seatsDriver).toBe(1);
    expect(matchQuery.requirement.seatsPassenger).toBe(2);
  });

  it('should create a query with modified algorithm settings', () => {
    const matchRequest: MatchRequest = new MatchRequest();
    matchRequest.departure = '2023-04-01 12:00';
    matchRequest.waypoints = [
      {
        lat: 49.440041,
        lon: 1.093912,
      },
      {
        lat: 50.630992,
        lon: 3.045432,
      },
    ];
    matchRequest.algorithm = AlgorithmType.CLASSIC;
    matchRequest.strict = true;
    matchRequest.useProportion = true;
    matchRequest.proportion = 0.45;
    matchRequest.useAzimuth = true;
    matchRequest.azimuthMargin = 15;
    matchRequest.remoteness = 20000;
    matchRequest.maxDetourDistanceRatio = 0.41;
    matchRequest.maxDetourDurationRatio = 0.42;
    const matchQuery: MatchQuery = new MatchQuery(
      matchRequest,
      defaultParams,
      mockGeorouterCreator,
      mockTimezoneFinder,
      mockTimeConverter,
    );
    expect(matchQuery.algorithmSettings.algorithmType).toBe(
      AlgorithmType.CLASSIC,
    );
    expect(matchQuery.algorithmSettings.restrict).toBe(Frequency.PUNCTUAL);
    expect(matchQuery.algorithmSettings.useProportion).toBeTruthy();
    expect(matchQuery.algorithmSettings.proportion).toBe(0.45);
    expect(matchQuery.algorithmSettings.useAzimuth).toBeTruthy();
    expect(matchQuery.algorithmSettings.azimuthMargin).toBe(15);
    expect(matchQuery.algorithmSettings.remoteness).toBe(20000);
    expect(matchQuery.algorithmSettings.maxDetourDistanceRatio).toBe(0.41);
    expect(matchQuery.algorithmSettings.maxDetourDurationRatio).toBe(0.42);
  });
});
